/*  code used from: edc_ecc.c	1.21 03/04/04 Copyright 1998-2002 Heiko Eissfeldt, Joerg Schilling */
/*
 * Copyright 1998-2002 by Heiko Eissfeldt
 * Copyright 2002 by Joerg Schilling
 *
 * This file contains protected intellectual property.
 *
 * reed-solomon encoder / decoder for compact discs.
 *
 */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; see the file COPYING.  If not, write to the Free Software
 * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*
The above header and the functions below are taken from the files:
edc_ecc.c
ecc.h
encoder_tables
in the libedc folder in the cdrtools-2.01.01a03.tar.bz2 package.

the rs_sub_rw_alog and rs_sub_rw_log arrays have been fixed because the original
ones are each missing a term

The orignal file is marked as GPL v2, and this new file is also marked GPL v2
*/


#include <windows.h>
#include <stdio.h>

// heres the code that seems to do the rs code generation correctly

#define RS_SUB_RW_BITS 6

#define PACKETS_PER_SUBCHANNELFRAME 4
#define LSUB_RAW 18
#define LSUB_QRAW 2
#define LSUB_Q 2
#define LSUB_P 4


__declspec(dllexport) int encode_LSUB_Q(unsigned char *);
__declspec(dllexport) int encode_LSUB_P(unsigned char *);
__declspec(dllexport) int decode_LSUB_Q(unsigned char *);
__declspec(dllexport) int decode_LSUB_P(unsigned char *);


/* ------------- tables generated by gen_encodes --------------*/
static unsigned char rs_sub_rw_alog[64] = {
1, 2, 4, 8,16,32, 3, 6,12,24,48,35,5,10,20,40,19,38,15,30,60,59,53,41,17,34, 7,14,28,56,51,37,
9,18,36,11,22,44,27,54,47,29,58,55,45,25,50,39,13,26,52,43,21,42,23,46,31,62,63,61,57,49,33,0} ; 
 
static unsigned char rs_sub_rw_log[64] = {
 0, 0, 1, 6, 2,12, 7,26, 3,32,13,35, 8,48,27,18, 4,24,33,16,14,52,36,54,9,45,49,38,28,41,19,56,
5,62,25,11,34,31,17,47,15,23,53,51,37,44,55,40,10,61,46,30,50,22,39,43,29,60,42,21,20,59,57,58};

static unsigned char SQ[2][2] = {
{26,6,},
{7,1,},
};
static unsigned char SP[4][20] = {
{57,38,44,29,17,57,53,58,60,39,12,38,18,41,6,25,39,37,5,18,},
{38,62,42,13,30,11,46,5,54,26,12,49,48,46,8,50,28,9,12,39,},
{32,18,41,49,52,62,38,36,39,58,37,24,34,51,51,27,28,36,22,21,},
{44,50,35,23,0,59,1,3,45,18,44,24,47,12,31,45,43,11,24,6,},
};




BOOL APIENTRY DllMain( HANDLE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved )
{
	switch (ul_reason_for_call)
		{
		case DLL_PROCESS_ATTACH:
			{
			//LoadDll();
			}
			break;

		case DLL_PROCESS_DETACH:
			{
			//UnloadDll();
			}
			break;

		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
			{

			}
			break;
		}
	return TRUE;
}


/* Generate Q parity symbols
 * parameters:
 * On input one pack of 24 bytes is expected.
 * On output one pack of 24 bytes is delivered.
 *
 * On encoding this function has to be run before encode_LSUB_P.
 */
__declspec(dllexport) int encode_LSUB_Q(unsigned char * inout)
{
	unsigned char *Q;
	int i;

	memmove(inout+LSUB_QRAW+LSUB_Q, inout+LSUB_QRAW, LSUB_RAW-LSUB_QRAW);
	Q = inout + LSUB_QRAW;

	memset(Q, 0, LSUB_Q);

	for (i = 0; i < LSUB_QRAW; i++) {
		unsigned char data;

		data = inout[i] & 0x3f;
		if (data != 0) {
			unsigned char base = rs_sub_rw_log[data];

			Q[0] ^= rs_sub_rw_alog[(base+SQ[0][i]) % (unsigned)((1 << RS_SUB_RW_BITS)-1)];
			Q[1] ^= rs_sub_rw_alog[(base+SQ[1][i]) % (unsigned)((1 << RS_SUB_RW_BITS)-1)];
		}
	}
	return (0);
}

__declspec(dllexport) int decode_LSUB_Q(unsigned char * inout)
{
	unsigned char Q[LSUB_Q];
	int i;

	memset(Q, 0, LSUB_Q);
	for (i = LSUB_QRAW + LSUB_Q -1; i>=0; i--) {
		unsigned char data;

		data = inout[LSUB_QRAW + LSUB_Q -1 -i] & 0x3f;
		if (data != 0) {
			unsigned char base = rs_sub_rw_log[data];

			Q[0] ^= rs_sub_rw_alog[(base+0*i) % (unsigned)((1 << RS_SUB_RW_BITS)-1)];
			Q[1] ^= rs_sub_rw_alog[(base+1*i) % (unsigned)((1 << RS_SUB_RW_BITS)-1)];
		}
	}
	return (Q[0] != 0 || Q[1] != 0);
}



/* Generate P parity symbols
 * parameters:
 * On input one tight packed pack of 20 bytes is expected.
 * On output one tight pack of 24 bytes is delivered.
 *
 * On encoding this function has to be run after encode_LSUB_Q.
 */
__declspec(dllexport) int encode_LSUB_P(unsigned char * inout)
{
	unsigned char *P;
	int i;

	P = inout + LSUB_RAW + LSUB_Q;

	memset(P, 0, LSUB_P);
	for (i = 0; i < LSUB_RAW + LSUB_Q; i++) {
		unsigned char data;

		data = inout[i] & 0x3f;
		if (data != 0) {
			unsigned char base = rs_sub_rw_log[data];

			P[0] ^= rs_sub_rw_alog[(base+SP[0][i]) % (unsigned)((1 << RS_SUB_RW_BITS)-1)];
			P[1] ^= rs_sub_rw_alog[(base+SP[1][i]) % (unsigned)((1 << RS_SUB_RW_BITS)-1)];
			P[2] ^= rs_sub_rw_alog[(base+SP[2][i]) % (unsigned)((1 << RS_SUB_RW_BITS)-1)];
			P[3] ^= rs_sub_rw_alog[(base+SP[3][i]) % (unsigned)((1 << RS_SUB_RW_BITS)-1)];
		}
	}
	return (0);
}



__declspec(dllexport) int decode_LSUB_P(unsigned char * inout)
{
	unsigned char P[LSUB_P];
	int i;

	memset(P, 0, LSUB_P);
	for (i = LSUB_RAW + LSUB_Q + LSUB_P-1; i>=0; i--) {
		unsigned char data;

		data = inout[LSUB_RAW + LSUB_Q + LSUB_P -1 -i] & 0x3f;
		if (data != 0) {
			unsigned char base = rs_sub_rw_log[data];

			P[0] ^= rs_sub_rw_alog[(base+0*i) % (unsigned)((1 << RS_SUB_RW_BITS)-1)];
			P[1] ^= rs_sub_rw_alog[(base+1*i) % (unsigned)((1 << RS_SUB_RW_BITS)-1)];
			P[2] ^= rs_sub_rw_alog[(base+2*i) % (unsigned)((1 << RS_SUB_RW_BITS)-1)];
			P[3] ^= rs_sub_rw_alog[(base+3*i) % (unsigned)((1 << RS_SUB_RW_BITS)-1)];
		}
	}
	return (P[0] != 0 || P[1] != 0 || P[2] != 0 || P[3] != 0);
}

